//注意：使用本脚本需设置好环境变量。各个环境变量名称及意义如下所示：
//|      变量名       |    含义    |               例子             |    备注      | 
//|  AccessKey        |  用户的AK  |118051d11de046ce8f6b8df42891****|    必填      |
//|  SecretKey        |  用户的SK  |47d4d29668ab48cf86d0a85f7f82****|    必填      |
//|  AuthVersion      |  认证版本  |                 1              |选填，默认为1 | 
//|ExpirationInSeconds|签名有效时间|               1800             |选填，默认1800|
//|   SignedHeaders   |  签名头域  |         host;x-bce-date        |选填
//                                                                  |默认将Host、
//                                                                  |Conten-Type、
//                                                                  |Content-Length、
//                                                                  |Content-MD5和以
//                                                                  |x-bce-开头的头域
//                                                                  |加入签名
//|    Timestamp      |   时间戳   |      2014-06-01T23:00:10Z      |选填          
//                                                                  |默认为当前时间|
function getTimestamp() {
    let timestamp = pm.environment.get("Timestamp");
    let now = timestamp ? new Date(timestamp.trim()) : new Date();
    return now.toISOString().replace(/\.\d+Z$/, 'Z');
}

function normalize(string, encodingSlash) {
    let kEscapedMap = {
        '!': '%21',
        '\'': '%27',
        '(': '%28',
        ')': '%29',
        '*': '%2A'
    };

    if (string === null) {
        return "";
    }
    var result = encodeURIComponent(string);
    result = result.replace(/[!'\(\)\*]/g, function($1) {
        return kEscapedMap[$1];
    });

    if (encodingSlash === false) {
        result = result.replace(/%2F/gi, '/');
    }

    return result;
}

function generateCanonicalUri() {
    url = pm.request.url;
    resources = url.path;
    if (!resources) {
        return "";
    }
    let normalizedResourceStr = "";
    for (let i = 0; i < resources.length; i++) {
        normalizedResourceStr += "/" + normalize(resources[i]);
    }
    return normalizedResourceStr;
}

function generateCanonicalQueryString() {
    url = pm.request.url;
    queryList = url.query.all();
    let normalizedQueryList = [];
    for (let i = 0; i < queryList.length; i++) {
        if (queryList[i].key.toLowerCase() == "authorization") {
            continue;
        }
        normalizedQueryList.push(normalize(queryList[i].key) + "=" + normalize(queryList[i].value));
    }
    normalizedQueryList.sort();
    return normalizedQueryList.join('&');
}

var g_signed_header = "";
function generateCanonicalHeaders() {
    let signedHeaders = pm.environment.get("SignedHeaders");
    let defaultHeaders = ["host", "content-length", "content-type", "content-md5"];
    let keyStrList = [];
    headerList = pm.request.headers;
    if (!headerList.has("host")) {
        var Header = require('postman-collection').Header;
        headerList.add(new Header({key: "host", value: pm.request.url.getHost()}));
    }
    
    if (!signedHeaders) {
        console.log("generateCanonicalHeader: using default headers");
        for (let i = 0; i < defaultHeaders.length; i++) {
            keyStrList.push(defaultHeaders[i]);
        }

        let headerListObj = headerList.all();
        for (let i = 0; i < headerListObj.length; i++) {
            let key = headerListObj[i].key;
            if (key.toLowerCase().startsWith("x-bce-")) {
                keyStrList.push(key.toLowerCase());
            }
        }
    } else {
        signedHeaders = signedHeaders.trim();
        keyStrList = signedHeaders.split(';');
        for (let i = 0; i < keyStrList.length; i++) {
            keyStrList[i] = keyStrList[i].toLowerCase();
        }
        if (!keyStrList.includes("host")) {
            keyStrList.push("host");
        }
    }
    let usedHeaderStrList = [];
    for (let i = 0; i < keyStrList.length; i++) {
        key = keyStrList[i];
        value = headerList.get(key);
        if (!value || value === "") {
            continue;
        }
        key = key.toLowerCase();
        value = value.trim();
        usedHeaderStrList.push(normalize(key) + ":" + normalize(value));
    }

    usedHeaderStrList.sort();
    let usedHeaderKeys = [];
    usedHeaderStrList.forEach(function(item) {
        usedHeaderKeys.push(item.split(':')[0]);
    });
    let canonicalHeaderStr = usedHeaderStrList.join('\n');
    g_signed_headers = usedHeaderKeys.join(';');
    return canonicalHeaderStr;
}

function generateAuthorization() {
    let timestamp = getTimestamp();
    let authVersion = pm.environment.get("AuthVersion");
    let expirationInSeconds = pm.environment.get("ExpirationInSeconds");
    let accessKey = pm.environment.get("AccessKey");
    let secretKey =  pm.environment.get("SecretKey");
    if (!authVersion) {
        console.log("generateAuthorization: using default AuthVersion");
    }
    if (!expirationInSeconds) {
        console.log("generateAuthorization: using default ExpirationInSeconds");
    }
    if (!accessKey) {
        console.log("generateAuthorization: AccessKey is empty!");
    }
    if (!secretKey) {
        console.log("generateAuthorization: SecretKey is empty!");
    }
    authVersion = (!authVersion) ? "1" : authVersion.trim();
    expirationInSeconds = (!expirationInSeconds) ? "1800" : expirationInSeconds.trim();
    signingKeyStr = "bce-auth-v" + authVersion + "/" + accessKey.trim() + "/" + timestamp + "/" + expirationInSeconds;
    signingKey = CryptoJS.HmacSHA256(signingKeyStr, secretKey.trim());
    console.log("signing key: " + signingKey.toString());

    canonicalUri = generateCanonicalUri();
    console.log("Canonical Uri: " + canonicalUri);

    canonicalQueryString = generateCanonicalQueryString();
    console.log("Canonical Query string: " + canonicalQueryString);

    canonicalHeaders = generateCanonicalHeaders();
    console.log("Canonical Headers: \n" + canonicalHeaders);

    method = pm.request.method;
    canonicalRequest = method.toUpperCase() + "\n" + canonicalUri + "\n" + canonicalQueryString + "\n" + canonicalHeaders;
    console.log("Canonical Request: \n" + canonicalRequest);

    signature = CryptoJS.HmacSHA256(canonicalRequest, signingKey.toString());
    console.log("Signature: " + signature.toString());

    Authorization = signingKeyStr + "/" + g_signed_headers + "/" + signature.toString();
    
    console.log("Authorization: " + Authorization);
    return Authorization;
}

pm.request.addHeader("Authorization:" + generateAuthorization());

